/*
MIT License

Copyright (c) 2021 Sulako

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/


#include "g25.h"


G25_::G25_(void) : HIDDevice(G25ReportDescriptor, sizeof(G25ReportDescriptor)){

  axes[AXIS_WHEEL]=new Axis_();
  axes[AXIS_ACC]=new Axis_();
  axes[AXIS_BRAKE]=new Axis_();
  axes[AXIS_CLUTCH]=new Axis_();

  axes[AXIS_ACC]->invert=true;
  axes[AXIS_BRAKE]->invert=true;
  axes[AXIS_CLUTCH]->invert=true;
}

void G25_::update(void) 
{
  G25Data data;

  //Отправка данных осей и кнопок
  data.Wheel=axes[AXIS_WHEEL]->value;
  data.Acc=axes[AXIS_ACC]->value;
  //data.Acc=127;
  
  data.Brake=axes[AXIS_BRAKE]->value;
  data.Clutch=axes[AXIS_CLUTCH]->value;

  data.buttons=buttons;

  //CombinedPedals
  if (data.Brake<64000)
    data.CombinedPedals=255-(data.Brake/512);
  else
    data.CombinedPedals=axes[AXIS_ACC]->value/512;
  
  SendReport(&data, sizeof(data));

  
  //Чтение и обработка команд обратной связи
  uint8_t ffdata[10]={0,0,0,0,0,0,0,0,0,0};
  int i;
  if (receive(ffdata, 10))
  {
    FFB.processData(ffdata);
      if (ffbInfo)
        FFB.info();
  }
  
}

/* 
 *  Обработка кнопок
 */
void G25_::setButtons(uint32_t btns)
{
   
  //первые 4 бита - кнопки крестовины (hat switch)
  uint8_t hat=(btns & 0xF);
  btns &= 0xFFFFFFF0;
  switch (hat)
  {
     case 0b00000001:
        btns |= HAT_UP;
        break;
     case 0b00000010:
        btns |= HAT_DOWN;
        break;
     case 0b00000100:
        btns |= HAT_LEFT;
        break;
    case 0b00001000:
        btns |= HAT_RIGHT;
        break;
     case 0b00000101:
        btns |= HAT_UP_LEFT;
        break;
     case 0b00001001:
        btns |= HAT_UP_RIGHT;
        break;
     case 0b00000110:
        btns |= HAT_DOWN_LEFT;
        break;
    case 0b00001010:
        btns |= HAT_DOWN_RIGHT;
        break;    
    default:
        btns |= HAT_CENTER;
  };

  //перемещение кнопок: 16->20, 17->16, 18->17, 19->18, 20->19
  bool tmp=bitRead(btns,23);
  bitWrite(btns, 23, bitRead(btns, 19));
  bitWrite(btns, 19, bitRead(btns, 20));
  bitWrite(btns, 20, bitRead(btns, 21));
  bitWrite(btns, 21, bitRead(btns, 22));
  bitWrite(btns, 22, tmp);
  
  buttons=btns;
}

int8_t G25_::getForce()
{
    return FFB.getValue(axes[AXIS_WHEEL]->value); 
}
