/*                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              /*
  MIT License

  Copyright (c) 2021 Sulako

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#include "SoftwareSerial.h"
SoftwareSerial mySerial(15, 16);
#include <digitalWriteFast.h>       //https://github.com/NicksonYap/digitalWriteFast
#include <avdweb_AnalogReadFast.h>  //https://github.com/avandalen/avdweb_AnalogReadFast
#include "fastPort.h"
#include "motor.h"
#include "g25.h"
#include <EEPROM.h>
#include "ffb.h"

int  ForceFeedback::wheelRange;

//=========== НАСТРОЙКИ ===========
// пины
#define ENC_A 0       // пин энкодера
#define ENC_B 1      // пин энкодера
#define INV_WHEEL 0
//Оси
#define PIN_ACC A1
#define PIN_BRAKE A2
#define PIN_CLUTCH A3
#define PIN_CTR A0  //кнопка установки нуля
#define STOP 30  //режим ограничения поворота руля мотором, значения от 0 до 127 максимальная.
//По умолчанию строки закомментированы в коде, так как требуется подборка под конкретную механику, мотор и бп.
//Смотри строку 161

//матрица кнопок
uint8_t btn_rows[] = { 4, 5, 6, 7, 8, 11, 12, 13, 14}; 
uint8_t btn_cols[] = {A4, A5, 2, 3};
volatile int encCounter = 0;
uint32_t timer; 
int tempMem;
int Mode = 2400;
G25_ G25;
FastPort fastPort;
Motor motor;
uint16_t timerFFB;
//отладка
uint16_t timerStat;
uint16_t countLoop, countFFB;
int8_t lastForce = 0;
bool showInfo = false;
bool setMode = false;
bool ffbInfo = false;
bool mode = false;
float wheelMaxTmp;
int wheelStop;
int wheelMax=2400;
int wheelRangeTmp=900;

void setup() {
  mySerial.begin(4800);
  EEPROM.get(0,Mode);
  uint8_t i;
  //настройка выводов
  pinMode(PIN_ACC, INPUT);
  pinMode(PIN_BRAKE, INPUT);
  pinMode(PIN_CLUTCH, INPUT);
  pinMode(PIN_CTR, INPUT_PULLUP);
  pinMode(ENC_A, INPUT_PULLUP);                           
  pinMode(ENC_B, INPUT_PULLUP);
  for (i = 0; i < 8; i++)
    pinMode(btn_rows[i], INPUT);
  for (i = 0; i < 4; i++)
    pinMode(btn_cols[i], INPUT_PULLUP);

  //Границы значений руля
  G25.axes[AXIS_WHEEL]->setRange(0, 65535);
  G25.axes[AXIS_ACC]->setRange(EEPROM.get(50, tempMem),EEPROM.get(60, tempMem));
  G25.axes[AXIS_BRAKE]->setRange(EEPROM.get(30, tempMem),EEPROM.get(40, tempMem));
  G25.axes[AXIS_CLUTCH]->setRange(EEPROM.get(10, tempMem),EEPROM.get(20, tempMem));
  
  fastPort.begin()                                                                      ;
  motor.begin();

  timerFFB = (uint16_t)micros();
  timerStat = (uint16_t)millis();
  
  // encoder pin on interrupt 2 (pin 0)
  attachInterrupt(2, encTick, FALLING);
}

void loop() {
  if(!(PINF & _BV (PF7))){
    encCounter=0;
  }
  
  gamepadTick();
  //Опрос осей 
  G25.axes[AXIS_ACC]->setValue(analogReadFast(PIN_ACC)); 
  G25.axes[AXIS_BRAKE]->setValue(analogReadFast(PIN_BRAKE)); 
  G25.axes[AXIS_CLUTCH]->setValue(analogReadFast(PIN_CLUTCH));

  //Опрос кнопок
  uint32_t btns = 0;
  uint32_t btnBit = 1;
  uint8_t row;
   for(row=0;row<8;row++)
    {
      fastPort.pinMode(btn_rows[row], OUTPUT);
      fastPort.digitalWrite(btn_rows[row], 0);
  
      if (!digitalReadFast(btn_cols[0]))
          btns |= btnBit;
      btnBit=btnBit<<1;
      
      if (!digitalReadFast(btn_cols[1]))
          btns |= btnBit;
      btnBit=btnBit<<1;
      
      if (!digitalReadFast(btn_cols[2]))
          btns |= btnBit;
      btnBit=btnBit<<1;
      
      if (!digitalReadFast(btn_cols[3]))
          btns |= btnBit;
      btnBit=btnBit<<1;
          
      fastPort.pinMode(btn_rows[row], INPUT);
    }
    G25.setButtons(btns);

    //передача и прием данных
    G25.update();
    countLoop++;

    //расчет обратной связи
    int8_t force;
    
   
   if ((!G25.FFB.fixedLoop) || ((uint16_t)(micros()-timerFFB)>1500))
    {
      timerFFB=micros();
      lastForce=force=G25.getForce();
      motor.setForce(force);
      countFFB++;
    }
  //Раскоментируй две строки ниже, если нужен ограничитель угла поворота.
// if(wheelStop==0){motor.setForce(STOP);}
 //if(wheelStop==-1){motor.setForce(-STOP);}   
 
  //отладка
if (showInfo)   
    if ((uint16_t)(millis() - timerStat) > 1000)
    {
      timerStat = millis();
      mySerial.println(
        " Buttons: " + String(btns) +
        " Wheel: " + String(G25.axes[AXIS_WHEEL]->rawValue) +
        " Acc: " + String(G25.axes[AXIS_ACC]->rawValue) +
        " Brake: " + String(G25.axes[AXIS_BRAKE]->rawValue) +
        " Clutch: " + String(G25.axes[AXIS_CLUTCH]->rawValue) +
        " Force: " + String(lastForce) +
        " loops/sec: " + String(countLoop) +
        " FFB/sec: " + String(countFFB)
      );
      countLoop = 0;
      countFFB = 0;
    }

  String buf;
  if (mySerial.available())
  {
    buf = mySerial.readString();
    
    if(setMode==true){
      setMode = !setMode;
      mySerial.print(F("Число тиков: "));
      EEPROM.put(0,(int)static_cast<float>(encCounter/2.5));
      mySerial.print(EEPROM.get(0,tempMem));
      mySerial.println(F(" записано в память"));
      buf="";
    }
    else
    {
    if (buf.indexOf("help") != -1) //Включение-выключение вывода данных по осям, силе ОС и времени обработки в Serial по команде info
    {
      mySerial.println(F("Список команд:\r"));
      mySerial.println(F(" info - Вкл/выкл вывода данных по осям, силе ОС"));
       mySerial.println(F(" ffb - Вкл/выкл вывода информации о FFB"));
       mySerial.println(F(" showCal - Вывод данных калибровки"));
       mySerial.println(F(" Команды калибровки:"));
        mySerial.println(F(" setTick - до ввода команды поверните руль влево до упора"));
        mySerial.println(F("  minC - сцепление отпущено"));
       mySerial.println(F("  maxC - сцепление нажато"));
        mySerial.println(F("  minB - тормоз отпущен"));
        mySerial.println(F("  maxB - тормоз нажат"));
        mySerial.println(F("  minA - газ отпущен"));
        mySerial.println(F("  maxA - газ нажат"));
        mySerial.println(F("  initMem - сброс памяти"));
        buf = "";
    }
    
    if (buf.indexOf("initMem") != -1){
      EEPROM.put(0, 2400);
      EEPROM.put(20, 1023);
      EEPROM.put(30, 0);
      EEPROM.put(40, 1023);
      EEPROM.put(50, 0);
      EEPROM.put(60, 1023);
      
      mySerial.println(F("Инициализация выполнена"));
      buf = "";
    }

      
    if (buf.indexOf("showCal") != -1){
      mySerial.println(F("Калибровочные данные педалей"));
      mySerial.println("Clutch: " + String(EEPROM.get(10,tempMem)) + " ----- " + String(EEPROM.get(20,tempMem)));
      mySerial.println("Brake:  " + String(EEPROM.get(30,tempMem)) + " ----- " + String(EEPROM.get(40,tempMem)));
      mySerial.println("Acc:    " + String(EEPROM.get(50,tempMem)) + " ----- " + String(EEPROM.get(60,tempMem)));
      mySerial.println("Mode:   " + String(EEPROM.get(0,tempMem)));   
      buf="";
    }
    
    if (buf.indexOf("minA") != -1){
      buf="";
      tempMem=analogRead(PIN_ACC);
      EEPROM.put(50, tempMem);
      mySerial.println(F("minA-OK"));     
    }
   if (buf.indexOf("minB") != -1){
      buf="";
      tempMem=analogRead(PIN_BRAKE);
      EEPROM.put(30, tempMem);
      mySerial.println(F("minB-OK"));     
    }
     if (buf.indexOf("minC") != -1){
      buf="";
      tempMem=analogRead(PIN_CLUTCH);
      EEPROM.put(10, tempMem);
      mySerial.println(F("minC-OK"));     
    }
    if (buf.indexOf("maxA") != -1){
      buf="";
      tempMem=analogRead(PIN_ACC);
      EEPROM.put(60, tempMem);
      mySerial.println(F("maxA-OK"));     
    }
   if (buf.indexOf("maxB") != -1){
      buf="";
      tempMem=analogRead(PIN_BRAKE);
      EEPROM.put(40, tempMem);
      mySerial.println(F("maxB-OK"));     
    }
     if (buf.indexOf("maxC") != -1){
      buf="";
      tempMem=analogRead(PIN_CLUTCH);
      EEPROM.put(20, tempMem);
      mySerial.println(F("maxC-OK"));     
    }
    if (buf.indexOf("info") != -1) //Включение-выключение вывода данных по осям, силе ОС и времени обработки в Serial по команде info
    {
      showInfo = !showInfo;
      countLoop = 0;
      countFFB = 0;
    }
    if (buf.indexOf("ffb") != -1) //Включение-выключение вывода информации об ОС по команде ffbw
    {
      G25.FFB.info();
    }
    }
    if (buf.indexOf("setTick") != -1){   
        setMode = !setMode;
        encCounter=0;
        mySerial.println(F("Поверните руль до упора вправо и введите end")); 
        buf="";
    }
  }

}

///проба пера

void encTick(){                                    // функция действие вызываемое прерыванием1
 if (PIND & _BV (PD3)) {   
      encCounter = encCounter + 1;          // по Ч.С.
    } 
    else {
      encCounter = encCounter - 1;          // против Ч.С
    }
    
}   

void gamepadTick() {
if (millis() - timer > 5) {
    //блок устанавливающий значение оси руля полученное из драйвера
    if(ForceFeedback::getWheelRange()!=wheelRangeTmp){
        wheelRangeTmp=ForceFeedback::getWheelRange();
        wheelMaxTmp=(Mode/2)*static_cast<float>((ForceFeedback::getWheelRange()/360.0));
        wheelMax=(int)wheelMaxTmp;
    }
    int wheel; 
    timer = millis();
    if (INV_WHEEL) wheel = constrain(-encCounter, -wheelMax, wheelMax);
    else wheel = constrain(encCounter, -wheelMax, wheelMax);
    wheel = map(wheel, -wheelMax, wheelMax, 0, 65535);
    wheelStop=wheel;
    G25.axes[AXIS_WHEEL]->setValue(wheel); 
  }
 
}
