/*
MIT License

Copyright (c) 2021 Sulako

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

#ifndef FFBFORCES_H
#define FFBFORCES_H

#include <stdint.h> 
#include <Arduino.h>
uint8_t KFactor(uint8_t offset, uint8_t K, bool hires=false);
int8_t convertForce(uint8_t f);

class FFB_Force
{
    public:
        bool active;
        virtual ~FFB_Force() {}
        FFB_Force():active(false) {}
        virtual int8_t getValue(unsigned int axis){return 0;};
        virtual void update(uint8_t* data, uint8_t slot){};
        virtual void info(){/*mySerial.print("none");*/};
};

//Все пружины: Spring, AutoCentering Spring, High Resolution Spring,High Resolution AutoCentering Spring
class FFB_Spring: public FFB_Force
{
    public:
        FFB_Spring(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t D1,D2,K1,K2,Clip;
        bool S1,S2,hires;
};

//Damper, High Resolution Damper, Friction
class FFB_Damper: public FFB_Force
{
    public:
        FFB_Damper(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t K1,K2,Clip,loops;
        int8_t value;
        unsigned int prevAxis;
        bool S1,S2,hires,friction,flagFirstRun;
};

class FFB_Variable: public FFB_Force
{
    public:
        FFB_Variable(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t T,S;
        bool D;
        uint8_t value, loops;

};

class FFB_Constant: public FFB_Force
{
    public:
        FFB_Constant(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        int8_t level;
};
class FFB_Sawtooth: public FFB_Force
{
    public:
        FFB_Sawtooth(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t L1,L2,T3;
        int8_t Inc;
        uint8_t value, loops;

};

class FFB_Trapezoid: public FFB_Force
{
    public:
        FFB_Trapezoid(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t L1,L2,T1,T2,T3,S;
        uint8_t value, loops, mode;

};
class FFB_Rectangle: public FFB_Force
{
    public:
        FFB_Rectangle(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t L1,L2,T1,T2,P;
        uint8_t value, loops;

};
class FFB_Ramp: public FFB_Force
{
    public:
        FFB_Ramp(uint8_t* data, uint8_t slot);
        void update(uint8_t* data, uint8_t slot);
        int8_t getValue(unsigned int axis);
        void info();
    private:
        uint8_t L1,L2,T,S;
        bool D;
        uint8_t value, loops;

};
#endif
