/*
MIT License

Copyright (c) 2021 Sulako 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/


#ifndef G25_H
#define G25_H

#include "HIDDevice.h"
#include "ffb.h"
#include "axis.h"

/* Оси */
#define AXIS_WHEEL 0
#define AXIS_ACC 1
#define AXIS_BRAKE 2
#define AXIS_CLUTCH 3

/* Направления крестовины */
#define HAT_UP 0b00000000
#define HAT_UP_RIGHT 0b00000001
#define HAT_RIGHT 0b00000010
#define HAT_DOWN_RIGHT 0b00000011
#define HAT_DOWN 0b00000100
#define HAT_DOWN_LEFT 0b00000101
#define HAT_LEFT 0b00000110
#define HAT_UP_LEFT 0b00000111
#define HAT_CENTER 0b00001000

extern bool ffbInfo;

/* HID дескриптор */
static const uint8_t G25ReportDescriptor[] PROGMEM = {
	0x05, 0x01,                     // USAGE_PAGE (Generic Desktop)  // 54
   0x09, 0x04,                     // USAGE (Joystick)
   0xa1, 0x01,                     // COLLECTION (Application)
   0xA1, 0x02,                     //     Collection (Logical),
   //0x85, 0x05,                     //     REPORT_ID (5)
   0x05, 0x01,                     //        Usage Page (Desktop),
   0x95, 0x01,                     //        Report Count (1 field),
   0x75, 0x10,                     //        Report Size (16 bit),
   0x14,                           //        Logical Minimum (0),
   0x26, 0xFF, 0xFF,               //        Logical Maximum (65535),
   0x34,                           //        Physical Minimum (0),
   0x46, 0xFF, 0xFF,               //        Physical Maximum (65535),
   0x09, 0x30,                     //        Usage (X),
   0x81, 0x02,                     //            >>>> Input (Variable), *** Wheel ***
   0x05, 0x01,                     //        Usage Page (Desktop),
   0x95, 0x01,                     //        Report Count (1 field),
   0x75, 0x04,                     //        Report Size (4 bit),
   0x25, 0x07,                     //        Logical Maximum (7),
   0x46, 0x3B, 0x01,               //        Physical Maximum (315),
   0x65, 0x14,                     //        Unit (Degrees),
   0x09, 0x39,                     //        Usage (Hat Switch),
   0x81, 0x42,                     //            >>>> Input (Variable), *** Hat ***
   0x95, 0x1C,                     //        Report Count (28 fields),
   0x75, 0x01,                     //        Report Size (1 bit),
   0x25, 0x01,                     //        Logical Maximum (1),
   0x45, 0x01,                     //        Physical Maximum (1),
   0x05, 0x09,                     //        Usage Page (Button),
   0x19, 0x01,                     //        Usage Minimum (01h),
   0x29, 0x1C,                     //        Usage Maximum (14h),
   0x81, 0x02,                     //            >>>> Input (Variable), *** Buttons ***
   0x05, 0x01,                     //        Usage Page (Desktop),
   0x95, 0x01,                     //        Report Count (1 field),
   0x75, 0x10,                     //        Report Size (16 bit),
   0x14,                           //        Logical Minimum (0),
   0x26, 0xFF, 0xFF,               //        Logical Maximum (65535),
   0x34,                           //        Physical Minimum (0),
   0x46, 0xFF, 0xFF,               //        Physical Maximum (65535),
   0x09, 0x32,                     //        Usage (Z),
   0x81, 0x02,                     //            >>>> Input (Variable), *** Accelerator ***
   0x05, 0x01,                     //        Usage Page (Desktop),
   0x95, 0x01,                     //        Report Count (1 field),
   0x75, 0x10,                     //        Report Size (16 bit),
   0x14,                           //        Logical Minimum (0),
   0x26, 0xFF, 0xFF,               //        Logical Maximum (65535),
   0x34,                           //        Physical Minimum (0),
   0x46, 0xFF, 0xFF,               //        Physical Maximum (65535),
   0x09, 0x35,                     //        Usage (Rz),
   0x81, 0x02,                     //            >>>> Input (Variable), *** Brake ***
   0x05, 0x01,                     //        Usage Page (Desktop),
   0x95, 0x01,                     //        Report Count (1 field),
   0x75, 0x10,                     //        Report Size (16 bit),
   0x14,                           //        Logical Minimum (0),
   0x26, 0xFF, 0xFF,               //        Logical Maximum (65535),
   0x34,                           //        Physical Minimum (0),
   0x46, 0xFF, 0xFF,               //        Physical Maximum (65535),
   0x09, 0x36,                     //        Usage (Slider),
   0x81, 0x02,                     //            >>>> Input (Variable), *** Clutch ***
   0x05, 0x01,                     //        Usage Page (Desktop),
   0x95, 0x01,                     //        Report Count (1 field),
   0x75, 0x08,                     //        Report Size (8 bit),
   0x14,                           //        Logical Minimum (0),
   0x26, 0xFF, 0x00,               //        Logical Maximum (255),
   0x34,                           //        Physical Minimum (0),
   0x46, 0xFF, 0x00,               //        Physical Maximum (255),
   0x09, 0x31,                     //        Usage (Y),
   0x81, 0x02,                     //            >>>> Input (Variable), *** Combined pedals ***
   0xC0,                           //    End Collection,
   0xA1, 0x02,                     //    Collection (Logical),
   0x09, 0x02,                     //        Usage (02h),
   0x95, 0x07,                     //        Report Count (7 fields),
   0x91, 0x02,                     //            <<<< Output (Variable), *** FFB & Control ***
   0xC0,                           //    End Collection, 
   0xC0                            //End Collection
};

//Данные устройства для отправки
typedef struct
{
  unsigned int Wheel;
  uint32_t buttons;
  unsigned int Acc;
  unsigned int Brake;
  unsigned int Clutch;
  uint8_t CombinedPedals;
} G25Data;



class G25_ : public HIDDevice
{
  public:
  	G25_();
    Axis_* axes[4];
    ForceFeedback FFB;
    void update(void);
    void setButtons(uint32_t btns);
    int8_t getForce();    
  private:
    uint32_t buttons;
};

#endif
